package controller;

import javafx.application.Platform;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import model.GameState;
import model.Player;

import java.io.*;

public class UpdateThread extends Thread
{
	Controller controller;
	BufferedReader gameReader;
	PrintWriter gameWriter;
	GameState gameState;
	Label statusLabel;

	public UpdateThread(Process program, Controller controller, GameState gameState, Label statusLabel)
	{
		this.gameReader = new BufferedReader(new InputStreamReader(program.getInputStream()));
		this.gameWriter = new PrintWriter(new OutputStreamWriter(program.getOutputStream()), true);
		this.gameState = gameState;
		this.statusLabel = statusLabel;
		this.controller = controller;
	}

	public void run()
	{
		boolean gameRunning = true;
		while (gameRunning)
		{
			try
			{
				String line = gameReader.readLine();
				System.out.println(line);
				if (line == null) //Normally this shouldn't happen (The game always end). So it is an error
				{
					gameRunning = false;
					Platform.runLater(() -> new Alert(Alert.AlertType.ERROR, "That's it! I rage quit!", ButtonType.FINISH).showAndWait());
				} else if (line.startsWith(Player.Red + " won") || line.startsWith(Player.Blue + " won") || line.startsWith("draw"))
				{
					gameRunning = false;
					Platform.runLater(() -> controller.gameEnd());
				} else if (line.contains("{")) //Line contains JSON
				{
					//gameState Update
					gameState.update(line.substring(line.indexOf("{"), line.lastIndexOf("}") + 1)); //Extract JSON string
					Platform.runLater(() -> controller.updateModelAndView());
					//If we can't play
					if (gameState.getCurrent_player().equals(gameState.getHumanPlayer()))
					{
						if (!gameState.getCanPlay(gameState.getHumanPlayer()))
						{
							Platform.runLater(() -> {
								statusLabel.setText("You can't play any move!");
								try
								{
									sleep(2000);
								}
								catch (InterruptedException e)
								{
									e.printStackTrace();
								}
								gameWriter.println("0"); //This pass the turn
							});
						}
					}

				}
				else if (line.startsWith("(") && line.contains("value:")) //lines with values (estimation) of the computer winning chances
				{ //We want to show a little message to the user depending on if we win or not
					Platform.runLater(() -> {
						try
						{
							float value = Float.valueOf(line.substring(line.indexOf("value:") + 7, line.indexOf(")")));
							if (value > 0.8)
								statusLabel.setText("Are you sure you don't want to give up?");
							else if (value > 0.3)
								statusLabel.setText("You may want to think harder...");
							else if (value > -0.3)
								statusLabel.setText("I will win, in the end");
							else if (value > -0.8)
								statusLabel.setText("You may think you are winning... but you are not really");
							else
								statusLabel.setText("I can not believe it... Human too strong... Segmentation fault.");
						}
						catch (Exception e) //This is not a core function, so if there is an exception we just ignore it
						{
							e.printStackTrace();
						}
					});
				}
			} catch (IOException e)
			{
				gameRunning = false;
				Platform.runLater(() ->
				{
					Alert alert = new Alert(Alert.AlertType.ERROR, "Error during reading from penguin program!", ButtonType.FINISH);
					alert.showAndWait();
					e.printStackTrace();
					Platform.exit();
				});
			} catch (Throwable e)
			{
				gameRunning = false;
				Platform.runLater(() -> {
					Alert alert = new Alert(Alert.AlertType.ERROR, "Unhandled exception in update thread: " + e.getMessage(), ButtonType.FINISH);
					alert.showAndWait();
					e.printStackTrace();
					Platform.exit();
				});
			}
		}
	}
}
